@@ -631,7 +631,7 @@ refresh_cache_file (GnomeBG     *bg,
 	GdkPixbufFormat *format;
 	gchar           *format_name;
 
-	if ((num_monitor != -1) && (width > 300) && (height > 300))
+	if ((num_monitor == -1) || (width <= 300) || (height <= 300))
 		return;
 
 	cache_filename = get_wallpaper_cache_filename (bg->filename, num_monitor, bg->placement, width, height);
@@ -1094,7 +1094,7 @@ gnome_bg_create_surface (GnomeBG	    *bg,
 		
 		pixbuf = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8,
 					 width, height);
-		gnome_bg_draw (bg, pixbuf, gdk_window_get_screen (GDK_DRAWABLE (window)), is_root);
+		gnome_bg_draw (bg, pixbuf, gdk_window_get_screen (window), is_root);
 		gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
 		g_object_unref (pixbuf);
 	}
@@ -1278,6 +1278,11 @@ fit_factor (int from_width, int from_height,
 	return MIN (to_width  / (double) from_width, to_height / (double) from_height);
 }
 
+/**
+ * gnome_bg_create_thumbnail:
+ *
+ * Returns: (transfer full): a #GdkPixbuf showing the background as a thumbnail
+ */
 GdkPixbuf *
 gnome_bg_create_thumbnail (GnomeBG               *bg,
 		           GnomeDesktopThumbnailFactory *factory,
@@ -1499,14 +1504,13 @@ gnome_bg_set_surface_as_root (GdkScreen *screen, cairo_surface_t *surface)
  * gnome_bg_set_surface_as_root_with_crossfade:
  * @screen: the #GdkScreen to change root background on
  * @surface: the cairo xlib surface to set root background from
- * @context: a #GMainContext or %NULL
  *
  * Set the root pixmap, and properties pointing to it.
  * This function differs from gnome_bg_set_surface_as_root()
  * in that it adds a subtle crossfade animation from the
  * current root pixmap to the new one.
  *
- * Return value: a #GnomeBGCrossfade object
+ * Return value: (transfer full): a #GnomeBGCrossfade object
  **/
 GnomeBGCrossfade *
 gnome_bg_set_surface_as_root_with_crossfade (GdkScreen       *screen,
@@ -1926,10 +1930,13 @@ ensure_timeout (GnomeBG *bg,
 {
 	if (!bg->timeout_id) {
 		double timeout = get_slide_timeout (slide);
-		bg->timeout_id = g_timeout_add_full (
-			G_PRIORITY_LOW,
-			timeout * 1000, on_timeout, bg, NULL);
 
+		/* G_MAXUINT means "only one slide" */
+		if (timeout < G_MAXUINT) {
+			bg->timeout_id = g_timeout_add_full (
+				G_PRIORITY_LOW,
+				timeout * 1000, on_timeout, bg, NULL);
+		}
 	}
 }
 
@@ -2876,16 +2883,24 @@ read_slideshow_file (const char *filename,
 	g_markup_parse_context_free (context);
 
 	if (show) {
+		int len;
+
 		t = mktime (&show->start_tm);
 
 		show->start_time = (double)t;
 			
 		dump_bg (show);
 
+		len = g_queue_get_length (show->slides);
+
 		/* no slides, that's not a slideshow */
-		if (g_queue_get_length (show->slides) == 0) {
+		if (len == 0) {
 			slideshow_unref (show);
 			show = NULL;
+		/* one slide, there's no transition */
+		} else if (len == 1) {
+			Slide *slide = show->slides->head->data;
+			slide->duration = G_MAXUINT;
 		}
 	}
 
@@ -2996,9 +3011,15 @@ gnome_bg_changes_with_time (GnomeBG *bg)
 	return FALSE;
 }
 
-/* Creates a thumbnail for a certain frame, where 'frame' is somewhat
+/**
+ * gnome_bg_create_frame_thumbnail:
+ *
+ * Creates a thumbnail for a certain frame, where 'frame' is somewhat
  * vaguely defined as 'suitable point to show while single-stepping
- * through the slideshow'. Returns NULL if frame_num is out of bounds.
+ * through the slideshow'.
+ *
+ * Returns: (transfer full): the newly created thumbnail or
+ * or NULL if frame_num is out of bounds.
  */
 GdkPixbuf *
 gnome_bg_create_frame_thumbnail (GnomeBG			*bg,
